/*******************************************************************************
**                    Boyer-Moore String Matching Algorithm                   **
**                                                                            **
**    Turbo-C version for MS-DOS                                              **
**    adapted from UNIX source code; amazingly, only a couple of minor        **
**      changes were required to make it Turbo-compatible. I was impressed.   **
**                                                                            **
**    Thomas A. Lundin                                                        **
**    Graphics Unlimited, Inc.                                                **
**    3000 Second Street North                                                **
**    Minneapolis, MN 55411                                                   **
**    (612) 588-7571                                                          **
**                                                                            **
*******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <dos.h>
#include <dir.h>
#include <io.h>
#include <stdlib.h>
#include <sys\stat.h>

#include <string.h>
#include "bm.h"
#include "Extern.h"

char s_path[80];                    /* source path */
char s_drv[3];                      /* drive */
char s_dir[66];                     /* directory */
char s_file[9];                     /* filename */
char s_ext[5];                      /* file ext */

struct ffblk fcb;                   /* where our file names will be stored */

main(argc,argv)
int argc;
char *argv[];
{
   /* grep based on Boyer-Moore algorithm */
   char BigBuff[MAXBUFF + 2];
   /*
   * We leave one extra character at the beginning and end of the buffer,
   * but don't tell Execute about it. This is so when someone is
   * scanning the buffer and scans past the end (or beginning)
   * we are still technically in the buffer (picky, but there ARE
   * machines which would complain)
   */
   int ret = 1, /* return code from Execute */
      NotFound = 0,     /* non-zero if file not readable */
      NFiles,
      NPats; /* number of patterns to search for */
   char *FlagPtr,
      **OptPtr; /* used to scan command line */
   int TextFile /* file to search */;
   struct PattDesc *DescVec[MAXPATS];
   int i;            /* index of filename argument */
   int first_time_through = TRUE;   /* whether to use findfirst or findnext */

   --argc;
   i = argc;
   OptPtr = argv + 1;
   while ( argc && **OptPtr == '-') {
      FlagPtr = *OptPtr + 1;
      do { 
         switch (*FlagPtr) {
            case 'c': cFlag = 1; break;
            case 'e': eFlag = 1;
               /* get the patterns from next arg */
               NPats = MkDescVec(DescVec,*++OptPtr);
               --argc;
               break;
            case 'f': fFlag = 1; 
               /* read the patterns from a file */
               NPats = GetPatFile(*++OptPtr,DescVec);
               --argc;
               break;
            case 'l': lFlag = 1; break;
            case 'n': nFlag = 1; break;
            case 's': sFlag = 1; break;
            case 'x': xFlag = 1; break;
            case 'h': hFlag = 1; break;
            case 'm': mFlag = 0; break;
            case 't': tFlag = 1; break;
            default:
               fprintf(stderr,
               "bm: invalid option: -%c \n",
               *FlagPtr);
               PutUsage();
               exit(2);
         } /* switch */
         ++FlagPtr;
      } while (*FlagPtr);
      ++OptPtr; --argc;
   } /* while */
   /* OptPtr now points to patterns */
   if (!fFlag && !eFlag) {
      if (!argc) {
         fprintf(stderr,"bm: no pattern specified\n");
         PutUsage();
         exit(2);
      } else
         NPats = MkDescVec(DescVec,*OptPtr);
      ++OptPtr; --argc;
   }
   /* OptPtr now points to first file */
   NFiles = argc;
   fnsplit(argv[i],s_drv,s_dir,s_file,s_ext);      /* split the names */
   if (!NFiles)
      ret &= Execute(DescVec,NPats,0,BigBuff+1);
   else {
      while (TRUE) {       /* a BREAK will end this loop */
         if (first_time_through) {
            if (findfirst(argv[i],&fcb,0) == EOF) {
               printf("No files matching %s\n",argv[i]);
               exit(1);
            }
            first_time_through = FALSE;
         }
         else {
            if (findnext(&fcb) == EOF)
               break;      /* no more files to convert */
         }
         FileName = fcb.ff_name;
         strcpy(s_path,s_drv);
         strcat(s_path,s_dir);
         strcat(s_path,fcb.ff_name);
         TextFile = _open(s_path,O_RDONLY);
         ret &= Execute(DescVec,NPats,TextFile,BigBuff+1);
         if (sFlag && !ret)
            exit(0);
         _close(TextFile);
      }
   }
   if (cFlag) printf("%d\n",MatchCount);
   exit(NotFound ? 2 : ret);
} /* main */
